unit h_eventlog;

interface

{$ifdef FPC}
  {$mode OBJFPC}
{$endif}

uses
  Classes, SysUtils;

{clear logfile - you should call it before initializing log,
  if you want to start a new log}
procedure ClearLog(const fn:string);

{initialize log - call it before using log}
procedure InitLog(const fn:string; const _strings:TStrings);

{WriteLog(s) - you shoul use it only in procs and funcs, not in methdos}
procedure WriteLog(const s:string=''); overload; inline;

{WriteLog(aobj,s) - you should use it in methods}
procedure WriteLog(const aobj:TObject; const s:string); overload; inline;

{Use it to write strings; aobj can be nil}
procedure WriteLog(const aobj:TObject; const ss:TStrings); overload; inline;

function GetHEventlogStrings:TStrings;
procedure setHEventlogStrings(const astrings:TStrings);

type
  TWritelogProcedure=procedure(const s:string);
  TWritelogMethod=procedure(const s:string) of object;

{SetWritelogMethod
  to add an optional message log handler (for example, TMemo)}
procedure SetWritelogMethod(const amethod:TWritelogMethod); inline;
procedure SetWritelogProc(const aproc:TWritelogProcedure); inline;

var
  except_onLogErrors:Boolean;

implementation

const
  const_hw=': ';

var
  lss:TStrings;
  lfo:boolean;
  t:text;
  wlmeth:TWritelogMethod;
  wlproc:TWriteLogProcedure;

procedure ClearLog(const fn:string);
begin
  if FileExists(fn) then DeleteFile(fn);
end;

procedure InitLog(const fn:string; const _strings:TStrings);
begin
  assignFile(t,fn);
  {
  Rewrite(t); lfo:=true;
  }
  try
    if FileExists(fn) then Append(t) else Rewrite(t);
    lfo:=true;
  except
    lfo:=false;
  end;
  lss:=_strings;
end;

procedure HandleError(const s:string);
begin
  if
    (pos(UpCase(' err'),UpCase(s))>1)
    or(pos(UpCase('err'),UpCase(s))=1)
  then
  begin
    //ShowMessage(s);
    raise Exception.Create(s);
  end;
end;

procedure WriteLog(const s:string); overload;
begin
  HandleError(s);
  if lfo then
  begin
    WriteLN(t,s);
    flush(t);
  end;
  if assigned(lss) then
  try
    lss.Add(s);
  except end;
  if assigned(wlmeth) then
  try
    wlmeth(s);
  except end;
  if assigned(wlProc) then
  try
    wlproc(s);
  except end;
end;

procedure WriteLog(const aobj:TObject; const s:string);
var
  pfs:string;
begin
  if assigned(aobj) then pfs:=aobj.ClassName;
  if not assigned(aobj) then pfs:='Nil';
  WriteLog(pfs+const_hw+s);
end;

procedure WriteLog(const aobj:TObject; const ss:TStrings);
var
  i:integer;
begin
  if not assigned(ss) then
  begin
    WriteLog(aobj,'Unassigned strings');
    exit;
  end;
  WriteLog(aobj,'TStrings:: ('+IntToStr(ss.Count)+')');
  for i:=0 to ss.Count-1 do
  begin
    WriteLog(IntToStr(i+1)+const_hw+ss[i]);
  end;
end;

function GetHEventlogStrings:TStrings;
begin
  result:=lss;
end;

procedure setHEventlogStrings(const astrings: TStrings);
begin
  lss:=astrings;
end;

procedure SetWritelogMethod(const amethod: TWritelogMethod);
begin
  wlmeth:=amethod;
end;

procedure SetWritelogProc(const aproc:TWritelogProcedure); inline;
begin
  wlproc:=aproc;
end;

initialization
  lss:=nil;
  wlmeth:=nil;
  wlproc:=nil;
  except_onLogErrors:=false;
end.

