//hmutex - bruteforce source code

{$ifdef hall}
  {$define hface}
{$endif}

{$ifdef hface}
//face here

const
  const_maxMutexesCount=100;
  const_mutexerr_mutexBusy=-1;
  const_mutexerr_mutexEmpty=-2;
  const_mutexerr_nomutex=-3;
  const_mutexerr_accessd=-4; //dostup zapreshen
  const_mutexerr_mutexAlreadyFree=-5;
  const_mutexerr_unknownErr=-666;

type
  TMutexIdx=longint;
  TMutexErrcode=longint;

  TMutex=record
    wholds:TAddress;
    mutobj:Taddress;
  end;
  
  PMutex=^TMutex;
  
  TMutexArray=array[0..const_maxMutexesCount]of TMutex;

{$ifdef hall}{$else}

  {is aMutex empty}
function isEmptyMutex(const aMutex:TMutex):boolean;

  {is aMutex free}
function isFreeMutex(const aMutex:TMutex):boolean;

  {is aMutex free and not empty}
function isGoodMutex(const aMutex:TMutex):boolean;

  {addres of current mutex owner}
function GetMutexOwner(const mutexID:TMutexIdx):TAddress;

  {find aMutObj in table, returns const_mutexerr_nomutex if not found}
function FindMutexByObj(const aMutObj:TAddress):TMutexIdx;

  {find mutex, which are hold by aWholds}
function FindMutexByOwner(const aWholds:TAddress):TMutexIdx;

  {Give me[=addr] mutex[=mutexID], please; returns mutex's object}
function GivemeMutex(const mutexId:TMutexIdx; const addr:TAddress):TAddress;

  {I finished working with mutex}
function ReleaseMutex(const mutexID:TMutexIdx; const addr:TAddress):TAddress;

  {create a new mutex and place it to table; returns false,
  if there are no free places}
function CreateMutex(const aMutObj:TAddress):boolean;

  {destroy a mutex; 
  returns false if mutex does not exists or already empty}
function DestroyMutex(const aMutIdx:TMutexIdx):boolean;

  {releases addr's mutexes
  returns count of released mutexes}
function ReleaseMyMutexes(const addr:TAddress):longint;

  {returns total count of existing mutexes}
function GetMutexesCount:longint;

{$endif}

{$endif}

{$ifdef hall}
  {$undef hface}
{$endif}

{$ifdef hface}{$else}
//impl here

const
  const_maxmut=const_maxMutexesCount;
  
var var_mutexes:TMutexArray;

function isEmptyMutex(const aMutex:TMutex):boolean;
begin
  isEmptyMutex:=(aMutex.mutobj=0);
end;

function isFreeMutex(const aMutex:TMutex):boolean;
begin
  isFreeMutex:=(aMutex.wholds=0)and( not isEmptyMutex(aMutex) );
end;

function isGoodMutex(const aMutex:TMutex):boolean;
begin
  isGoodMutex:=isFreeMutex(aMutex) and (not isEmptyMutex(aMutex));
end;

function mutexes(const aidx:TMutexIdx):PMutex;
begin
  mutexes:=nil;
  if aidx<0 then exit;
  if aidx>const_maxmut then exit;
  mutexes:=@(var_mutexes[aidx]);
end;

function GetMutexOwner(const mutexID:TMutexIdx):TAddress;
var
  m:PMutex;
begin
  m:=mutexes(mutexID);
  if m=nil then 
  begin
    GetMutexOwner:=const_mutexerr_nomutex;
    exit;
  end;
  if isEmptyMutex(m^) then
  begin
    GetMutexOwner:=const_mutexerr_mutexEmpty;
    exit;
  end;
  GetMutexOwner:=m^.wholds;
end;

function FindMutexByObj(const aMutObj:TAddress):TMutexIdx;
var
  i:TMutexIdx;
begin
  for i:=0 to const_maxmut do
  begin
    if mutexes(i)^.mutobj=aMutObj then 
    begin
      FindMutexByObj:=i;
      exit;
    end;
  end;
  FindMutexByObj:=const_mutexerr_nomutex;
end;

function FindMutexByOwner(const aWholds:TAddress):TMutexIdx;
var
  i:TMutexIdx;
begin
  for i:=0 to const_maxMut do
  begin
     if (mutexes(i)^.wholds=aWholds)
       and(not isEmptyMutex(mutexes(i)^)) then
     begin
       FindMutexByOwner:=i;
       exit;
     end;
  end;
  findMutexByOwner:=const_mutexerr_nomutex;
end;

function GivemeMutex(const mutexId:TMutexIdx; const addr:TAddress):TAddress;
var
  m:PMutex;
begin
  GivemeMutex:=const_mutexerr_unknownErr;
  m:=mutexes(mutexId);
  if m=nil then 
  begin
    GivemeMutex:=const_mutexerr_nomutex;
    exit;
  end;
  if isEmptyMutex(m^) then 
  begin
    GivemeMutex:=const_mutexerr_mutexEmpty;
    exit;
  end;
  if not isFreeMutex(m^) then
  begin
    GivemeMutex:=const_mutexerr_mutexBusy;
    exit;
  end;
  //if success then we v here v
  with m^ do
  begin
    wholds:=addr;
    GivemeMutex:=mutobj;
  end;
end;

function ReleaseMutex(const mutexID:TMutexIdx; const addr:TAddress):TMutexErrcode;
var
  m:PMutex;
begin
  m:=mutexes(mutexID);
  if m=nil then 
  begin
    ReleaseMutex:=const_mutexerr_nomutex;
    exit;
  end;
  if isEmptyMutex(m^) then
  begin
    ReleaseMutex:=const_mutexerr_mutexEmpty;
    exit;
  end;
  if isFreeMutex(m^) then 
  begin
    ReleaseMutex:=const_mutexerr_mutexAlreadyFree;
    exit;
  end;
  case GetMutexOwner(mutexId)=addr of
    true:
    begin
      releaseMutex:=0;
      m^.wholds:=0;
    end;
    false: ReleaseMutex:=const_mutexerr_accessd;
  end;
  ReleaseMutex:=0;
end;

function CreateMutex(const aMutObj:TAddress):boolean;
var
  i:TMutexIdx;
begin
  i:=FindMutexByObj(0);
  if i=const_mutexerr_nomutex then 
  begin
    CreateMutex:=false;
    exit;
  end;
  with mutexes(i)^ do
  begin
    mutobj:=aMutObj;
    wholds:=0;
  end;
  CreateMutex:=true;
end;

function DestroyMutex(const aMutIdx:TMutexIdx):boolean;
var
  m:PMutex;
begin
  m:=mutexes(aMutIdx);
  if isEmptyMutex(m^) then
  begin
    DestroyMutex:=true;
    exit;
  end;
  case isFreeMutex(m^) of
    true:
    begin
      m^.wholds:=0;
      m^.mutobj:=0;
      DestroyMutex:=true;
    end;
    false:
    begin
      DestroyMutex:=false;
    end;
  end;
end;

function ReleaseMyMutexes(const addr:TAddress):longint;
var
  count:longint;
  i:TMutexIdx;
begin
  count:=0;
  repeat
    i:=FindMutexByOwner(addr);
    if i=const_mutexerr_nomutex then break;
    if ReleaseMutex(i,addr)=0 then
    begin
      inc(count);
    end;
  until false;
  ReleaseMyMutexes:=count;
end;

function GetMutexesCount:longint;
var
  i,count:longint;
begin
  count:=0;
  for i:=0 to const_maxmut do
  begin
    if not isEmptyMutex(mutexes(i)^) then inc(count);
  end;
  GetMutexesCount:=count;
end;

{$endif}